import { Reducer } from 'redux';
export interface RuleOption<T> {
    key: string;
    message: string | ((_: T) => string);
    validate?: ((_: T) => boolean);
    defaultValue?: T;
}
export interface FormField<T> {
    vaild: boolean;
    message: string;
    value: T;
}
export declare type FormAction<T> = (_?: T) => {
    type: string;
    payload?: T;
};
export declare type FormActions = {
    fill: FormAction<any>;
    reset: FormAction<any>;
    [key: string]: FormAction<any>;
};
export declare class Rule<T> {
    key: string;
    message: string | ((_: T) => string);
    validate: ((_: T) => boolean);
    defaultValue: T;
    constructor(options: RuleOption<T>);
    toField(value?: T): FormField<T>;
}
export interface FormModelOptions {
    rules: Rule<any>[];
    name: string;
}
export declare type FormReduxReducer<S, D> = (state: S, value?: D) => S;
export declare class FormModel<State> {
    rules: Rule<any>[];
    name: string;
    defaultState: State;
    reducers: {
        [key: string]: FormReduxReducer<any, any>;
    };
    constructor({ rules, name }: FormModelOptions);
    initReducer(): void;
    handle: Reducer;
    actions(): FormActions;
}
