"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var constants_1 = require("./constants");
var util_1 = require("./util");
function only(rules) {
    var _this = this;
    var result = {};
    Object.keys(this).forEach(function (key) {
        var rule = rules.filter(function (r) { return r.key === key; })[0];
        if (rule) {
            result[key] = rule.toField(_this[key]);
        }
    });
    return result;
}
var Rule = /** @class */ (function () {
    function Rule(options) {
        var message = options.message, validate = options.validate, key = options.key;
        this.key = key;
        this.message = message || key + " \u6821\u9A8C\u5931\u8D25";
        this.validate = validate || (function (data) { return (typeof data === 'boolean' || Boolean(data)); });
    }
    Rule.prototype.toField = function (value) {
        value = value || this.defaultValue || null;
        var message;
        if (typeof this.message === 'function') {
            message = this.message(value);
        }
        else {
            message = this.message;
        }
        var vaild = this.validate(value);
        return {
            value: value,
            vaild: vaild,
            message: vaild ? '' : message,
        };
    };
    return Rule;
}());
exports.Rule = Rule;
var FormModel = /** @class */ (function () {
    function FormModel(_a) {
        var rules = _a.rules, name = _a.name;
        var _this = this;
        this.handle = function (state, action) {
            state = state || _this.defaultState;
            var handler = _this.reducers[action.type];
            if (handler) {
                return handler(state, action.payload);
            }
            return state;
        };
        this.rules = rules || [];
        this.name = name;
        this.defaultState = rules.reduce(function (pre, rule) {
            pre[rule.key] = rule.toField();
            return pre;
        }, {});
        this.initReducer();
    }
    FormModel.prototype.initReducer = function () {
        var _this = this;
        this.reducers = {};
        this.rules.forEach(function (rule) {
            _this.reducers[constants_1.prefix + "/" + _this.name + "/CHANGE_FIELD_" + util_1.toUnderscore(rule.key)] = function (state, value) {
                var _a;
                return (__assign({}, state, (_a = {}, _a[rule.key] = rule.toField(value), _a)));
            };
        });
        this.reducers[constants_1.prefix + "/" + this.name + "/RESET"] = function (state) { return (__assign({}, state, _this.defaultState)); };
        this.reducers[constants_1.prefix + "/" + this.name + "/FILL"] = function (state, value) { return (__assign({}, state, only.call(value, _this.rules))); };
    };
    FormModel.prototype.actions = function () {
        var _this = this;
        var actions = this.rules.reduce(function (pre, rule) {
            pre[rule.key] = function (payload) { return ({
                type: constants_1.prefix + "/" + _this.name + "/CHANGE_FIELD_" + util_1.toUnderscore(rule.key),
                payload: payload,
            }); };
            return pre;
        }, {});
        actions.reset = function () { return ({
            type: constants_1.prefix + "/" + _this.name + "/RESET",
        }); };
        actions.fill = function (payload) { return ({
            type: constants_1.prefix + "/" + _this.name + "/FILL",
            payload: payload,
        }); };
        return actions;
    };
    return FormModel;
}());
exports.FormModel = FormModel;
//# sourceMappingURL=FormModel.js.map